% ---------------------------------------------------------------------------
%  Programs for Appendix C
%  Quarterly Baseline Program for the NBER Macroeconomics Annual
%  June 2006
% ---------------------------------------------------------------------------

clear all; close all;  format compact;
      
global paraminnovar;

%----------------------- NIPA consumption and price data  -------
    data = xlsread('NIPA2_3_5.xls');
    kill = isnan(data(:,1));
    keep = find(kill==0);
    data=data(keep,3:end)';

    totalexp=data(:,1);
    ndurableexp=data(:,6);
    serviceexp=data(:,13);

    data = xlsread('NIPA2_3_4.xls');
    kill = isnan(data(:,1));
    keep = find(kill==0);
    data=data(keep,3:end)';

    ptotal=data(:,1);
    pndurable=data(:,6);
    pservice=data(:,13);

    data = xlsread('NIPA2_3_3.xls');
    kill = isnan(data(:,1));
    keep = find(kill==0);
    data=data(keep,3:end)';

    qtotal=data(:,1);
    qndurable=data(:,6);
    qservice=data(:,13);

    wndurable=mean(ndurableexp./(qndurable.*pndurable));
    wservice=mean(serviceexp./(qservice.*pservice));

    w  = [wndurable wservice];
    PI =[pndurable pservice];
    QI =[qndurable qservice];
    expend =  [ ndurableexp serviceexp];
    T = length(PI);

    priceindex(1)=1;
    qtyindex(1)=1;

for i=2:T  
    priceindex(i)=priceindex(i-1)*sqrt((sum(w.*PI(i,:).*QI(i-1,:))*sum(w.*PI(i,:).*QI(i,:)))/...
        (sum(w.*PI(i-1,:).*QI(i-1,:))*sum(w.*PI(i-1,:).*QI(i,:))));
    qtyindex(i)=qtyindex(i-1)*sqrt((sum(w.*PI(i-1,:).*QI(i,:))*sum(w.*PI(i,:).*QI(i,:)))/...
        (sum(w.*PI(i-1,:).*QI(i-1,:))*sum(w.*PI(i,:).*QI(i-1,:))));
end

    cgrowth = diff(log(qtyindex))';
    pgrowth = diff(log(priceindex))';
    sam = (1947.25:.25:2005.75)';

    % start in 1952
    beg = 21;    
    cgrowth = cgrowth(beg:end); 
    pgrowth = pgrowth(beg:end);
    
    sam = sam(beg:end);

%----------------------------Population-----------------------------

    % Population numbers from different sources look different!
    
    % FRED provides monthly numbers, growth rates do not look correct
    
    % Here, go with quarterly numbers from NIPA Table 2.1
    
     data = xlsread('NIPA2_1.xls');
     kill = isnan(data(:,1));
     keep = find(kill==0);
     data=data(keep,3:end)';  
     pop = data(1:end-1,38);
     popgrowth = diff(log(pop));
     popgrowthnipa = popgrowth(beg:end);
    
    % Original population numbers, Bureau of Labor Statistics
    % recommended by McGratten 
    
     load population.dat;
     pop= reshape(population(:,2:end-1)',58*12,1);
     pop= (pop(1:3:end)+pop(2:3:end)+pop(3:3:end))/3;
     popgrowth2 = diff(log(pop)); 
     popgrowth2 = popgrowth2(17:end);
     popgrowth = popgrowth2;
         
    % figure shows that series look VERY different 
    % both series look screwy
    % large outliers around the turn of the decades
    % perhaps this has to do with the census, 
    % which is done every decade ???
    % e.g., NIPA series has 1960:Q1 as clear outlier

    figure;
    subplot(2,1,1);
    plot(sam, popgrowthnipa*400,'LineWidth',2);
    axis([sam(1) sam(end) 0 4]);
    title('Population growth measured with data from NIPA tables');
    subplot(2,1,2);
    plot(sam, popgrowth*400,'LineWidth',2);
    axis([sam(1) sam(end) 0 4]);
    title('Population growth measured with original BLS data');
    
    print -depsc2 population.eps;  
    

    % BLS population growth
    % cgrowth = cgrowth - popgrowth;
   
    % NIPA propulation growth
     cgrowth = cgrowth -popgrowthnipa;
    
    growthA = cgrowth*400;
    pgrowthA = pgrowth*400;

%-------------Other inflation measures--------------------------%

   load CPIAUCSL.txt;
   cpi = CPIAUCSL(3:3:end,4);
   pgrowth2 = diff(log(cpi));
   pgrowth2 = pgrowth2(beg:end);
   %pgrowth = pgrowth2;
       
   load GNPDEF.txt;
   gnp = GNPDEF(:,4);
   pgrowth3 = diff(log(gnp));
   pgrowth3 = pgrowth3(beg:end);
   %pgrowth = pgrowth3;
      
   load PCECTPI.txt;
   pce = PCECTPI(:,4);
   pgrowth3 = diff(log(pce));
   pgrowth3 = pgrowth3(beg:end);
   %pgrowth = pgrowth3;   


   figure;
   subplot(2,1,1);
   plot(sam, 400*pgrowth, 'k', sam, 400*pgrowth2,'g','LineWidth',2);
   axis([sam(1) sam(end) -1 4*4.5]);
   title('Inflation measured with CPI');
   subplot(2,1,2);
   plot(sam, 400*pgrowth, 'k', sam, 400*pgrowth3, 'g','LineWidth',2);
   axis([sam(1) sam(end) -1 4*4.5]);
   title('Inflation measured with GNP Deflator');
   print -depsc2 inflationmeasures.eps;



%-----------------Data on nominal yields------------------------------------%

    load zc.dat;
    ydat = zc(1:3:end,2:end); % quarterly data, starts 1952:2
 
    load short.dat;
    short = short(319:end,:);
    short = short(1:3:end,2);
    
    ydat = [short ydat]/4;

    ydatA = ydat*4;
    
%----------------Estimate model--------------------------------
       
 % small numbers (in percent)
    mu_c = mean(cgrowth);
    mu_pi= mean(pgrowth);
    mu_z = [mu_c; mu_pi];
    z=[cgrowth pgrowth] - ones(length(cgrowth),1)*mu_z';

 % estimate in big numbers
    data  = 100*z;
   
 % starting values, always start with the independent case, 
 % add correlations later
 

 % x0 estimated as parameters?
 
    E=0;
 
     if E == 0

 % x0 = 0 

      c0 = [0.5443   -0.0992    0.2804    1.0188    0.2416   -0.1172    0.0887    0.5262...
            0.4315   -0.0920    0.2929]';

     else
     
 % x0 estimated as parameters
      c0 = [0.5137   -0.1047    0.2548    1.0131    0.2457   -0.1143    0.1012    0.5218 ...
            0.4242   -0.0846    0.2911    1.0699   -0.7376]'; 

     end

 % early estimations: set MaxIter to 5000
    options = optimset('TolX',1e-5,'TolFun',1e-5,'Display','iter','MaxIter',5000);
    paraminnovar = 'paraminno';

    [c0,FVAL, EXITFLAG]=fminsearch('innoMLEobj',c0,options,data,E);

    disp('how did the search algorithm end?')
    disp(EXITFLAG);
    
    [c0,FVAL,EXITFLAG,OUTPUT,GRAD,HESSIAN]= fminunc('innoMLEobj',c0,options,data,E);
   
    FVALUNR = FVAL;
    
%---------------standard errors ----------------------------------
    
    T = length(data)-1;

    disp('function value');
    disp(-T*FVAL);

    disp('estimate   standard error')

    % variance is (1/T) * information matrix
    % information matrix = - 1/T * hessian of the log-likelihood function 
    % log-likelihood function = sum of log densities

    % HESSIAN = information matrix = average of (- log densities)

    varbeta1 = inv(HESSIAN)/T;
    v1 = diag(varbeta1);
    v1 = sqrt(v1);

    % information matrix = 1/T  * sum of product of densities
    % G = vector of gradients 
    % cov = covariance of vector of gradients, 
    %       computes 1/T * sum (df(t)/da)(df(t)/da)
    G=gradp('innodensity',c0,data,E);

    varbeta2 = inv(cov(G))/T;
    v2 = diag(varbeta2);
    v2 = sqrt(v2);

    disp([c0 v1 v2])
  
    
    CHOL = chol(varbeta1)';   % save for computing standard errors for nonlinear stuff

   
%--------------Comparison with VAR ------------

    paraminnovar = 'paramvar';

  
    if E == 0
       
    c0var = [0.2784   -0.1457    0.1455    0.8837    0.4324   -0.0937    0.3166];
         
    else
        
    c0var = [0.2784   -0.1457    0.1455    0.8837    0.4259   -0.0864    0.3148   -4.4214 ...
            26.2383    1.0956   -0.7279]'; 
    
    end
    [c0var,FVAL, EXITFLAG]=fminsearch('innoMLEobj',c0var,options,data,E);
     
    disp('how did the search algorithm end?')
    disp(EXITFLAG);
    [c0var,FVAL,EXITFLAG,OUTPUT,GRAD,HESSIAN]= fminunc('innoMLEobj',c0var,options,data,E);

    FVALR = FVAL;
    
    disp('Likelihood-ratio test statistic')
    -2*T*(FVALUNR - FVALR)
    disp('chi2(4) 5% and 1% critical values'); 
    disp(chi2inv([0.95 0.99],4)); 

%----------------Match Parameters-----------------------------------------%

paraminnovar = 'paraminno';

    % estimated big numbers
    [C, A, AK, omega] = feval(paraminnovar,c0);

    sigma = chol(omega)';   % standard dev of dc and pi shocks
    [x] = state(c0,data,E);   % recover state x(t) for t = 1,...,T 
    
    % RETURN TO SMALL NUMBERS
    % Estimated Z = 100*z, got Z(t) = CX(t)+e(t)
    %           X = 100*x, got X(t) = AX(t-1)+AKe(t)
    % Transfrom back           z(t) = (CX(t)+e(t))/100
    %                          z(t) = Cx(t) + e(t)/100
    % need to sigma/100
    % Also                     x(t) = Ax(t-1)+ AKe(t)/100
    % don't need to adjust
    
    
     x = x/100;
     sigma = sigma/100;
 
     T= length(x);
     [T,N] = size(x);
 
     phi_x = A;
     sig_x = AK*sigma;
 
     phi_c   = [1 0];
     phi_pi  = [0 1];
     sig_c   = sigma(1,:);
     sig_pi  = sigma(2,:);
     
%----------------------Compute yield curve -----------------------

   
     m = ssmodel([mu_c;mu_pi],phi_x,sig_x,eye(2),sigma,eye(2),'full sample');


%----------------------Comparison with VAR ----------------------
      Z = [cgrowth pgrowth];
     [T,N] = size(Z);
      Z = Z - ones(T,1)*mean(Z);
 
     rhv = [ones(T-1,1) Z(1:T-1,:)];
     bvar = rhv\Z(2:T,:) ; 
     muvar = bvar(1,:)';
     phivar = bvar(2:end,:)';
     err = Z(2:T,:)-rhv*bvar;
     sigvar = err'*err/T;
     sigvar = chol(sigvar)';
     
     disp('eigenvalues from 2nd order VAR')
     eig(phivar*phivar)
 
     mvar = ssmodel([mu_c;mu_pi],phivar,sigvar,eye(2),zeros(2,2),eye(2),'var');
     
     names{1} = 'consumption';
     names{2} = 'inflation';
     array{1} = mvar;
     
     summaryA(array,data/100,[mu_c mu_pi],names,10);

         
     names{1} = '\Deltac';
     names{2} = '\pi';
     covplot(m*100,data,names,10);
    
     % picture with BLS data
     % print -depsc2 covapp.eps;

     % picture with NIPA data
      print -depsc2 covappnipa.eps

%----------------Forecasts of consumption growth and inflation  -------
    
    [guess,errors] = predict(m,x',40);
    guess = guess*100;


hors = [1 4 20];
colors = 'bcryk';
a = figure;
for i=1:2;     
        
    subplot(2,1,i);
    hold on;
    
    plot(sam,data(:,i),'g-','LineWidth',2);
    legends{1} = 'Data';
    for horind = 1:length(hors);
        hornow = hors(horind);
        style = [colors(horind) '-'];
        plot(sam(hornow+1:end),squeeze(guess(i,hornow+1:length(sam),hornow)),style,'LineWidth',2);
       legends{horind+1} = ['Horizon ' num2str(hornow)];
    end;
    title(names{i});

    if i==1;
    legend(legends);
    end;
end;

%-------------------IR plots ----------------------------------


    mmatlab = idss(A,[],C,[],AK,'NoiseVariance',omega);

    sigA = 4*chol(omega)';

    for i=0:20;
        predc(i+1,:)  = phi_c*(phi_x^i)*AK;
        predpi(i+1,:) = phi_pi*(phi_x^i)*AK;
    end;

    predc = [1 0;predc];
    predc = predc(1:21,:);
    predpi = [0 1; predpi];
    predpi = predpi(1:21,:);


%-----------------Standard errors for IR  -------------------------

   S = 1000; 
   % S=10000;  % picture for paper is based on 10000 simulations

    seed(100); disp('# simulations'); disp(S);
     
    for s=1:S

       err = randn(length(c0),1);
       c0h = c0+ CHOL*err;
      
       [Ch, Ah, AKh, omegah] = feval(paraminnovar,c0h);

       sigmah = chol(omegah)';   % standard dev of dc and pi shocks
       sigmah = sigmah/100;
       sigxh = AKh*sigmah;

       if  max(abs(eig(Ah))) < 1 
     
             covh(s,:,:,:) = covfunction(Ah,Ch,sigxh,sigmah,10);

             for i=0:20;
                  predch(s,i+1,:) = Ch(1,:)*(Ah^i)*AKh;
                  predpih(s,i+1,:) = Ch(2,:)*(Ah^i)*AKh;
                  predcomh(s,i+1,:) = predpih(s,i+1,:)+predch(s,i+1,:);
             end;

       
       else 
             exits(s,:) = 1;
       end
    end

    keep = find(exits<1);
    
    covh = (100^2)*covh(keep,:,:,:); 
    
    [S,I,J,M] = size(covh);
    
    predch = predch(keep,:,:);
    predpih = predpih(keep,:,:);
    predcomh = predcomh(keep,:,:);
    for i = 0:20
        for j = 1:2
         predcstd(:,i+1,j) = std(predch(:,i+1,j));
         predpistd(:,i+1,j) = std(predpih(:,i+1,j)); 
         predcomstd(:,i+1,j) = std(predcomh(:,i+1,j));
        end
    end
    
    for i=1:I
        for j=1:J
            for m=1:M

            covsort(:,i,j,m)  = sort(covh(:,i,j,m));
            covconf(:,i,j,m)  = percentile(squeeze(covsort(:,i,j,m)), [0.05 0.95]);
            end
        end
    end

%--------------------- Plot IR and Standard errors ----------------
    figure;
    subplot(3,2,1);
        plot([0:20], predc(1:21,1),'k', ...
             [0:20], predc(1:21,1)+[0;2*squeeze(predcstd(1,1:20,1))'],'r:', ...
             [0:20],  predc(1:21,1)-[0;2*squeeze(predcstd(1,1:20,1))'],'r:', 'Linewidth',2);

    grid on;
    title('Change in \Deltac forecast to \Deltac surprise');
    axis([0 20 -.2 1.01]);

    subplot(3,2,2);
    plot([0:20], predc(:,2),'k', ...
         [0:20], predc(:,2)+[0;2*squeeze(predcstd(1,1:20,2))'],'r:', ...
        [0:20], predc(:,2)-[0;2*squeeze(predcstd(1,1:20,2))'],'r:', 'Linewidth',2);
    grid on;
    title('Change in \Deltac forecast to \pi surprise');
    axis([0 20 -0.2 0.05]);

    subplot(3,2,3);

    plot([0:20],predpi(1:21,1),'k',  ... 
         [0:20], predpi(:,1)+[0;2*squeeze(predpistd(1,1:20,1))'],'r:', ...
         [0:20], predpi(:,1)-[0;2*squeeze(predpistd(1,1:20,1))'],'r:', 'Linewidth',2);
    title('Change in \pi forecast to \Deltac surprise');

    grid on;
    axis([0 20 0 0.4]);

    subplot(3,2,4);

    plot([0:20],predpi(1:21,2),'k',  ... 
         [0:20],predpi(:,2)+[0;2*squeeze(predpistd(1,1:20,2))'],'r:', ...
         [0:20], predpi(:,2)-[0;2*squeeze(predpistd(1,1:20,2))'],'r:', 'Linewidth',2);
    title('Change in \pi forecast to \pi surprise');
    grid on;
    axis([0 20 0 1]);

    subplot(3,2,5);

    plot([0:20],(predpi(1:21,1)+predc(1:21,1)),'k', ...
         [0:20], predpi(:,1)+predc(:,1)+[0;2*squeeze(predcomstd(1,1:20,1))'],'r:', ...
         [0:20], predpi(:,1)+predc(:,1)-[0;2*squeeze(predcomstd(1,1:20,1))'],'r:', 'Linewidth',2);
    grid on;
    xlabel('time (in quarters)');
    title('Change in \Deltac+\pi forecast to \Deltac surprise');

    axis([0 20 0 1]);

    subplot(3,2,6);

    plot([0:20],(predpi(1:21,2)+predc(1:21,2)),'k', ...
         [0:20], predpi(:,2)+predc(:,2)+[0;2*squeeze(predcomstd(1,1:20,2))'],'r:', ...
         [0:20], predpi(:,2)+predc(:,2)-[0;2*squeeze(predcomstd(1,1:20,2))'],'r:', 'Linewidth',2);
    grid on;
    xlabel('time (in quarters)');

    title('Change in \Deltac+\pi forcast to \pi surprise');
    axis([0 20 0 1.01]);

  print -depsc2 ir.eps;


%----------------------Compute yield curve -----------------------


%^^^^^^^^^^^^^^^^^ different models and yields to be computed

    % preference parameters   
        numprefs = 3;    
        betas = 1.005*ones(1,5);
        gams = [1 40 110];
    % maturities    
        mats = [1 4 8 12 16 20]';
        numyields = max(mats);
        keepyields = length(mats);
        %smaller model with short rate and 5-1 spread
        mats2 = [20];
        keepyields2 = 1+length(mats2);

    % arrays for results   
        yieldmodels = cell(numprefs,1);
        yieldchanges = cell(numprefs,1);
        compyieldmodels = cell(numprefs,1);
        ynoms = zeros(T,numyields,numprefs);
        yreals = zeros(T,numyields,numprefs);
   
%------------ Negative slope with power utility model, arbitrary gamma------------------    
    

    j = 20;
    GAM = [0:0.5:20];

    for i = 1:length(GAM) 
    
        gam = GAM(i);
        X(i) = -(gam*sig_c+sig_pi)*sig_x'*sumA(phi_x,j-1)'*([gam;1]);
        Xc(i) = -(gam*sig_c)*sig_x'*sumA(phi_x,j-1)'*[gam;0];
        Xcpi(i) = -(gam*sig_c)*sig_x'*sumA(phi_x,j-1)'*[0;1];
        Xpic(i) = -(sig_pi)*sig_x'*sumA(phi_x,j-1)'*[gam;0];
        Xpi(i) = -(sig_pi)*sig_x'*sumA(phi_x,j-1)'*[0;1];
    end

    figure;

    plot(GAM,X);
    figure;
    plot(GAM, 400*Xc, 'k', GAM, 400*Xcpi, 'k:', GAM, 400*Xpic, 'k-.', ...
         GAM, 400*Xpi, 'k--', 'Linewidth', 2);
    hold on;
    plot(GAM, zeros(length(GAM),1), 'k');
     
    text(12,-.4, '- \gamma^2 cov( \Deltac, future \Deltac )' );
    text(12,-.15, '- \gamma cov( \Deltac, future \pi )');
    text(12,-.045, '- cov( \pi, future \pi )');
    text(12, .12, '- \gamma cov( \pi, future \Deltac )');
 
  %  ylabel('Percent Excess Return');    
    xlabel('Coefficient of relative risk aversion \gamma');
    print -depsc2 covterms.eps;

 %---------------------  Yield curve implications -----------------   
    
    param = [1.005; 40];
    [param0] =fminsearch('prefparamobj',param,options,ydat, x, mu_c, phi_c, sig_c, mu_pi, phi_pi, sig_pi, phi_x, sig_x);
    
    disp('beta gamma');
    disp(param0);
    
    numprefs = 3;
    betas = param0(1)*ones(1,numprefs);
    gams = [1 param0(2) 110];
          
% loop over models for computation of yields    
for jj=1:numprefs;
    
    beta = betas(jj);
    betaprime = min([beta;1]);
    gam = gams(jj);
     
    % real pricing kernel       
    fbeta=sig_c + betaprime*phi_c* inv(eye(2)-phi_x*betaprime) * sig_x;
   
    mu_m  = log(beta)-mu_c-0.5*((1-gam)^2)*fbeta*fbeta';
    phi_m = -phi_c;
    sig_m = -sig_c+(1-gam)*fbeta; 
      
    % real yields
    [ar,br] = findAB(phi_x, sig_x, mu_m, phi_m, sig_m, 20);
    [ar,br] = annualfindaby(ar,br,20);
    yreal = ones(T,1)*ar'+x*br';
    yreal = yreal*100;
    
% nominal yields
    mu_mn  = mu_m  - mu_pi;
    phi_mn = phi_m - phi_pi;
    sig_mn = sig_m - sig_pi; 

    [a,b] = findAB(phi_x, sig_x, mu_mn, phi_mn, sig_mn, 20);
    [a,b] = annualfindaby(a,b,20);
    ynom = ones(T,1)*a'+x*b';
    ynom = ynom*100;
    
    yreals(:,:,jj) = yreal;

    ynoms(:,:,jj) = ynom;
    
    Epi = (mu_pi+x*phi_pi')*100;
    zline = zeros(length(sam),1);

    curname = ['\beta = ' num2str(beta) ', \gamma = ' num2str(gam)];

    % construct stacked models of funds and nominal yields

    % fundamentals + all yields
%     mallyields = ssmodel([mu_c;mu_pi;a;ar],phi_x,sig_x,...
%                  [eye(2);b*phi_x;br*phi_x],[sigma;b*sig_x;br*sig_x],eye(2),curname);
%     mnomyields = ssmodel([mu_c;mu_pi;a],phi_x,sig_x,[eye(2);b*phi_x],[sigma;b*sig_x],eye(2),curname);
        % fundamentals + all yield (nominal) changes
    mnomchanges = ssmodel([mu_c;mu_pi;zeros(keepyields,1)],phi_x,sig_x,[eye(2);b(mats,:)*(phi_x-eye(2))],...
                                [sigma;b(mats,:)*sig_x],eye(2),curname);
    % baseline model: fundamentals + nominal short rate + nominal 5-1 spread                        
    mallyields = ssmodel([mu_c;mu_pi;a(mats);ar(mats)],phi_x,sig_x,...
                  [eye(2);[b(mats,:);br(mats,:)]*phi_x],...
                       [sigma;[b(mats,:);br(mats,:)]*sig_x],eye(2),curname);
    mnomyields = ssmodel([mu_c;mu_pi;a(mats)],phi_x,sig_x,[eye(2);b(mats,:)*phi_x],[sigma;b(mats,:)*sig_x],eye(2),curname);

    myields = ssmodel([mu_c;mu_pi;a(1);a(mats2)-a(1)*ones(keepyields2-1,1)],phi_x,sig_x,...
                  [eye(2);b(1,:)*phi_x;(b(mats2,:)-ones(keepyields2-1,1)*b(1,:))*phi_x],...
                       [sigma;b(1,:)*sig_x;(b(mats2,:)-ones(keepyields2-1,1)*b(1,:))*sig_x],eye(2),curname);
    yieldmodels{jj} = myields;
    compyieldmodels{jj} = mnomyields;
    yieldchanges{jj} = mnomchanges;
    
    
end;
    
    ynomsA = ynoms*4;
    yrealsA =yreals*4;

    ynomAEU = squeeze(ynomsA(:,:,1));
    yrealAEU = squeeze(yrealsA(:,:,1));
    
    ynomA = squeeze(ynomsA(:,:,2));
    yrealA = squeeze(yrealsA(:,:,2));
    

 % save factors and model yields for our discussant, pierpaolo benigno 
 % yben = [x ynomA(:,[1 3*4 5*4])];
 % save benigno.dat yben -ascii;
    
   
    figure;
    
    mature = [1 1*4 2*4 3*4 4*4 5*4];
    plot(mature, mean(ynomA(:,mature)), 'g', ...
         mature, mean(ydatA),'b', ...
         mature, mean(ynomAEU(:,mature)),'r','Linewidth',2); 
     xlabel('maturity in quarters');
     ylabel('yields in percent');
    text(13, 5.4, 'Benchmark Model');
    text(7, 6.1, 'Data');
     
    text(8,5.2, 'Expected Log Utility');
    %print -depsc2 averages.eps;

    
    figure;
    
    mature = [1 1*4 2*4 3*4 4*4 5*4];
    plot(mature, mean(yrealA(:,mature)), 'g', ...
         mature, mean(yrealAEU(:,mature)),'r','Linewidth',2); 
    xlabel('maturity in quarters');
    ylabel('yields in percent');
    axis([1 20 -2 6.4]);
    text(10, 3, 'Expected Log Utility');
    text(10, -1, 'Benchmark Model');
     %    print -depsc2 realaverages.eps;
    
   
%------------------Autocorrelations -----------------------------
    
    for i = 1:size(ynomA,2)
       autocmodel(i) =  autoc(ynomA(:,i),1,0);
    end
    autocmodel = autocmodel(mats);
    for i = 1:6
        autocdata(i) = autoc(ydatA(:,i),1,0);
    end
    
    disp('Autocorrelations of nom yields: data, model');
    disp([autocdata; autocmodel])
    
    
    for i = 1:size(ynomA,2)
       autocmodel(i) =  autoc(yrealA(:,i),1,0);
    end
    autocmodel = autocmodel(mats);
   
    disp('Autocorrelations of real yields: data, model');
    disp([autocmodel])
    
%------------------------------------------------------------
    
    load ydatmodel.dat;
    
    yrealRICHA = 4*ydatmodel(:,1:20);
    ynomRICHA = 4*ydatmodel(:,21:end);

    figure;
    plot(sam,ynomRICHA(:,1),'b',sam,pgrowthA,'y',sam,ydatA(:,1),'k',sam,ynomA(:,1),'c','Linewidth',1.4)
    print -depsc2 rich2.eps;

    
%-------------------------Plot shocks ----------------------------    
    
    zshocks = 100*(z(2:end,:)-x(1:end-1,:)*C);
    
    numa=1;
    
    numb = 4;
    numb = ones(numb,1)*(1/numb);
    
   % PRINT THIS PICTURE MANUALLY TO SQUEEZE VERTICALLY
    figure;
    plot(sam(2:end),zeros(length(sam(2:end)),1),'k', ...
         sam(2:end),abs(filter(numb, numa, zshocks(:,1))),'g', ...
         sam(2:end), abs(filter(numb,numa,zshocks(:,2))),'k','Linewidth',1.5);
    text(1972,0.58, 'Inflation Surprises');
    text(1955,0.4, 'Consumption Surprises');
   % text(1956,0.35,'Surprises');
    axis tight;
    print -depsc2 shocks.eps;
   % no longer in paper because of 50 page limit
   
%-----------------Data on nominal yields------------------------------------%

    spreaddat = ydat(:,6)-ydat(:,1);
   
    basedata = [data (ydat(:,1)-mean(ydat(:,1))) (spreaddat-mean(spreaddat))];
    
    spreaddatA = spreaddat*4;
    dataA = data*4;
    basedataA = basedata*4;
    
    figure;
    subplot(2,2,1);
    plot(sam,dataA(:,1));
    title('cons growth');
    axis tight;
    subplot(2,2,2);
    plot(sam,dataA(:,2));
    axis tight;
    title('inflation');
    subplot(2,2,3);
    plot(sam,ydatA(:,[1 5]));
    legend('1 year','5 year');
    axis tight;
    subplot(2,2,4);
    plot(sam,[ydatA(:,1) spreaddatA]);
    legend('1 year','spread');
    plot(sam,zeros(T,1),'k-');

    axis tight;
       
%----------------Compare Model and Data ---------------------------------%
disp('***************************************************************************');
disp('***************************************************************************');
disp('***************************************************************************');
disp(' =========================  BASELINE MODEL =============================');

datameans = [mu_c*100,mu_pi*100,mean(ydat(:,1)), mean(spreaddat)];

summaryA(yieldmodels,basedata/100,datameans/100,names,3);

disp('***************************************************************************');
disp('***************************************************************************');
disp('***************************************************************************');
disp(' =========================  ALL NOMINAL YIELDS =============================');

% summary statistics; all yields
names{1} = 'cons. growth';
names{2} = 'inflation';
for j=1:keepyields2;
    names{j+2} = ['nominal ' num2str(mats(j))];
   % names{j+numyields+2} = ['real ' num2str(mats(j))];
end;
datameans = [mu_c*100,mu_pi*100,mean(ydat)];
datad = [data ydat-ones(T,1)*mean(ydat)];
summaryA(compyieldmodels,datad/100,datameans/100,names,3);

% table with levels of nominal and real yields
disp('***************************************************************************');
disp('***************************************************************************');
disp('***************************************************************************');
disp(' =========================  REAL TIME YIELDS =============================');
    legends{1}='data'; 
    disp('data: mean and standard deviations');
    disp([mean(ydatA);std(ydatA)]);
    disp('models: mean and standard deviations');
    for jj=1:numprefs;           
     mod = yieldmodels{jj};   
     disp(mod.name);
     legends{jj+1}=mod.name;
     disp('^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^');
     disp([mean(squeeze(ynomsA(:,mats,jj)));std(squeeze(ynomsA(:,mats,jj)))]);     
     disp('      REAL: mean and standard deviations');
     disp([mean(squeeze(yrealsA(:,mats,jj)));std(squeeze(yrealsA(:,mats,jj)))]);
    end;

    colors = 'bcrmyk';

% real time short rate and long rate pictures
  for matt = 1:5:6;
    figure;
    hold on;
    plot(sam, ydatA(:,matt),'g-','LineWidth',2);
    for jj=1:numprefs;
    style = [colors(jj) '-'];    
    plot(sam, squeeze(ynomsA(:,mats(matt),jj)),style,'LineWidth',2);
    end;
    for jj=1:numprefs;
    style = [colors(jj) '--'];    
    plot(sam, squeeze(yrealsA(:,mats(matt),jj)),style,'LineWidth',2);
    end;
    title(['maturity ' num2str(mats(matt)) ' (nominal solid, real dashed)']);
    legend(legends);
  end;
    
 
 % real time spread
    spreadnoms = squeeze(ynoms(:,20,:)) - squeeze(ynoms(:,1,:));

    spreadnomsA = squeeze(ynomsA(:,20,:)) - squeeze(ynomsA(:,1,:));

    figure;
    hold on;
    plot(sam, spreaddatA,'g-','LineWidth',2);
    for jj=1:numprefs;
    style = [colors(jj) '-'];    
    plot(sam, spreadnomsA(:,jj),style,'LineWidth',2);
    end;
    title('nominal spread');
    legend(legends);
    plot(sam,zeros(T,1),'k-');

    figure;
    plot(sam, zline,'k', sam, ynomA(:,1)-yrealA(:,1), 'r', sam, Epi, 'b:');
    title('difference in yields and expected inflation');
    
    
    % model implied data said (only need one because we have demeaned it!)
    basedatamod = [data ([squeeze(ynoms(:,1,1)) squeeze(spreadnoms(:,1))] ...
            - ones(T,1)*mean([squeeze(ynoms(:,1,1)) squeeze(spreadnoms(:,1))]))];
    
    basedatamodA = basedatamod*4;    

    basedatayieldsA = [ydatA - ones(T,1)*mean(ydatA)];
    basedatamodyieldsA = 4*[(squeeze(ynoms(:,:,1))- ones(T,1)*mean(squeeze(ynoms(:,:,1))))];
  

%     figure; 
%     plot(sam, pgrowth*100,'k', yearF, piF*100,'r', sam+1, Epi,'r:', date-1, ydat(:,1), 'g', 'Linewidth',1.5);
%     % here, 1-year yield is pre-shifted, 
%     print -depsc2 fig1.eps;
%    
%     axis([1960 2005 1 14]);
%    
%     
    %figure; 
    %plot(yearF, piF(4:4:end)*100 - Epi,'Linewidth',1.5);

    %print -depsc2 fig2.eps;
   
    %axis([1960 2005 1 14]);
freq;


predictability;

save ex1yields ynoms yreals;    

