% ---------------------------------------------------------------------------
%  quarterly baseline program

% unrestricted 4 factor program
% ---------------------------------------------------------------------------
clear all; close all;  format compact;
      
%----------------------- NIPA consumption and price data  -------
    data = xlsread('NIPA2_3_5.xls');
    kill = isnan(data(:,1));
    keep = find(kill==0);
    data=data(keep,3:end)';

    totalexp=data(:,1);
    ndurableexp=data(:,6);
    serviceexp=data(:,13);

    data = xlsread('NIPA2_3_4.xls');
    kill = isnan(data(:,1));
    keep = find(kill==0);
    data=data(keep,3:end)';

    ptotal=data(:,1);
    pndurable=data(:,6);
    pservice=data(:,13);

    data = xlsread('NIPA2_3_3.xls');
    kill = isnan(data(:,1));
    keep = find(kill==0);
    data=data(keep,3:end)';

    qtotal=data(:,1);
    qndurable=data(:,6);
    qservice=data(:,13);

    wndurable=mean(ndurableexp./(qndurable.*pndurable));
    wservice=mean(serviceexp./(qservice.*pservice));

    w  = [wndurable wservice];
    PI =[pndurable pservice];
    QI =[qndurable qservice];
    expend =  [ ndurableexp serviceexp];
    T = length(PI);

    priceindex(1)=1;
    qtyindex(1)=1;

for i=2:T  
    priceindex(i)=priceindex(i-1)*sqrt((sum(w.*PI(i,:).*QI(i-1,:))*sum(w.*PI(i,:).*QI(i,:)))/...
        (sum(w.*PI(i-1,:).*QI(i-1,:))*sum(w.*PI(i-1,:).*QI(i,:))));
    qtyindex(i)=qtyindex(i-1)*sqrt((sum(w.*PI(i-1,:).*QI(i,:))*sum(w.*PI(i,:).*QI(i,:)))/...
        (sum(w.*PI(i-1,:).*QI(i-1,:))*sum(w.*PI(i,:).*QI(i-1,:))));
end

    cgrowth = diff(log(qtyindex))';
    pgrowth = diff(log(priceindex))';
    sam = (1947.25:.25:2005.75)';
   
    beg = 21;    % 1952
    
    cgrowth = cgrowth(beg:end);
    pgrowth = pgrowth(beg:end);
    sam = sam(beg:end);
    obs=length(sam);
%-----------------Data on nominal yields------------------------------------%

    load zc.dat;
    ydat = zc(1:3:end,2:end); % quarterly data, starts 1952:2
   
    load short.dat;
    short = short(319:end,:);
    short = short(1:3:end,2);
    
    ydat = [short ydat]/4;

    figure;
    plot(sam,ydat(:,end)-ydat(:,1));
    axis tight;

%----------------Estimate model--------------------------------
    
    
%  % small numbers (in percent)
%     mu_c = mean(cgrowth);
%     mu_pi= mean(pgrowth);
%     mu_z = [mu_c; mu_pi];
%     z=[cgrowth pgrowth] - ones(length(cgrowth),1)*mu_z';
% 
%     % estimate in big numbers
%     data  = 100*z;
   
    y1 = ydat(:,1);
    spreaddat = ydat(:,6)-ydat(:,1);   
    
% small numbers (in percent)
    mu_c = mean(cgrowth);
    mu_pi= mean(pgrowth);
    z=[cgrowth pgrowth] - ones(length(cgrowth),1)*[mu_c mu_pi];
% estimate in big numbers
    data0  = z*100;
    
    mu_y1 = mean(y1);
    mu_spread = mean(spreaddat);
    data0 = [data0 y1-mu_y1*ones(obs,1) spreaddat-mu_spread*ones(obs,1)];

    data = data0;
 
    E = 0;

    c0 = [ 0.6043    0.2564    0.1389   -0.0958   -0.0571    1.0423    0.1260   -0.0360 ... 
          -0.0075   -0.0273    0.9060    0.0233    0.1514   -0.0304   -0.0221    0.8829 ...
           0.2427    0.0699    0.1191   -0.0879   -0.0746    0.4403    0.0978   -0.0977  ...
          -0.2385    0.1424    0.7701    0.0426    0.0895   -0.1955    0.2861    0.5480 ...
           0.4223   -0.0815    0.2880    0.0306    0.0448    0.2336   -0.0129   -0.0166 ...
          -0.1117    0.1186]';

     options = optimset('TolX',1e-5,'TolFun',1e-5,'Display','iter','MaxIter',5000);

     [c0,FVAL, EXITFLAG]=fminsearch('innoMLEobj4',c0,options,data,E);
      
     disp('how did the search algorithm end?')
     disp(EXITFLAG);

     [c0,FVAL,EXITFLAG,OUTPUT,GRAD,HESSIAN]= fminunc('innoMLEobj4',c0,options,data,E);
     disp('function value');
     disp(-T*FVAL);

 
%---------------standard errors ----------------------------------
    
    T = length(data)-1;

    disp('estimate   standard error')

    % variance is (1/T) * information matrix
    % information matrix = - 1/T * hessian of the log-likelihood function 
    % log-likelihood function = sum of log densities

    % HESSIAN = information matrix = average of (- log densities)

    varbeta1 = inv(HESSIAN)/T;
    v1 = diag(varbeta1);
    v1 = sqrt(v1);

    % information matrix = 1/T  * sum of product of densities
    % G = vector of gradients 
    % cov = covariance of vector of gradients, 
    %       computes 1/T * sum (df(t)/da)(df(t)/da)
    G=gradp('innodensity4',c0,data,E);

    varbeta2 = inv(cov(G))/T;
    v2 = diag(varbeta2);
    v2 = sqrt(v2);

    disp([c0 v1 v2])
 
%----------------Match Parameters-----------------------------------------%
  
    % estimated big numbers
    [C, A, AK, omega] = paraminno4(c0);

    sigma = chol(omega)';   % standard dev of dc and pi shocks
    [x] = state4(c0,data,E);   % recover state

    Nstates = 4;
        % RETURN TO SMALL NUMBERS
    % Estimated Z = 100*z, got Z(t) = CX(t)+e(t)
    %           X = 100*x, got X(t) = AX(t-1)+AKe(t)
    % Transfrom back           z(t) = (CX(t)+e(t))/100
    %                          z(t) = Cx(t) + e(t)/100
    % need to sigma/100
    % Also                     x(t) = Ax(t-1)+ AKe(t)/100
    % don't need to adjust

     x = x/100;
     sigma = sigma/100;
% 
     T= length(x);
     [T,N] = size(x);
% 
     phi_x = A;
     sig_x = AK*sigma;
% 
     phi_c   = [1 0 0 0];
     phi_pi  = [0 1 0 0];
     sig_c   = sigma(1,:);
     sig_pi  = sigma(2,:);
     
%----------------------Compute yield curve -----------------------

    
    m = ssmodel([mu_c;mu_pi;mu_y1;mu_spread],phi_x,sig_x,eye(Nstates),sigma,eye(Nstates),'full sample');
    
    [guess,errors] = predict(m,zeros(4,1),10);

    names{1} = 'cons.';
    names{2} = 'infl.';
    names{3} = 'short rate';
    names{4} = 'spread';
    covplot(m*100,data,names,20);

    print -depsc2 largeinfopaper.eps;
    print -depsc2 figure6new.eps;
    
%----------------------Compute yield curve -----------------------
    param = [1.005; 40];
    [param0] =fminsearch('prefparamobj',param,options,ydat, x, mu_c, phi_c, sig_c, mu_pi, phi_pi, sig_pi, phi_x, sig_x);
    
    disp('beta gamma');
    disp(param0);
    
    betas = [1.0052 param0(1)*ones(1,2)];
    gams = [57.34 param0(2) 100];

%^^^^^^^^^^^^^^^^^ different models and yields to be computed
% preference parameters   
    numprefs = 3;    

% maturities    
    mats = [1 4 8 12 16 20]';
    numyields = max(mats);
    keepyields = length(mats);
    %smaller model with short rate and 5-1 spread
    mats2 = [20];
    keepyields2 = 1+length(mats2);

% arrays for results   
    yieldmodels = cell(numprefs,1);
    yieldmodels2 = cell(numprefs,1);
    yieldchanges = cell(numprefs,1);
    compyieldmodels = cell(numprefs,1);
    ynoms = zeros(T,numyields,numprefs);
    yreals = zeros(T,numyields,numprefs);

    % loop over models for computation of yields    
for jj=1:numprefs;
    
    beta = betas(jj);
    betaprime = min([beta;1]);
    gam = gams(jj);
     
    % real pricing kernel       
    fbeta=sig_c + betaprime*phi_c* inv(eye(4)-phi_x*betaprime) * sig_x;
   
    mu_m  = log(beta)-mu_c-0.5*((1-gam)^2)*fbeta*fbeta';
    phi_m = -phi_c;
    sig_m = -sig_c+(1-gam)*fbeta; 
      
    % real yields
    [ar,br] = findAB(phi_x, sig_x, mu_m, phi_m, sig_m, 20);
    [ar,br] = annualfindaby(ar,br,20);
    yreal = ones(T,1)*ar'+x*br';
    yreal = yreal*100;
    
% nominal yields
    mu_mn  = mu_m  - mu_pi;
    phi_mn = phi_m - phi_pi;
    sig_mn = sig_m - sig_pi; 

    [a,b] = findAB(phi_x, sig_x, mu_mn, phi_mn, sig_mn, 20);
    [a,b] = annualfindaby(a,b,20);
    ynom = ones(T,1)*a'+x*b';
    ynom = ynom*100;
    
    yreals(:,:,jj) = yreal;

    ynoms(:,:,jj) = ynom;
    
    Epi = (mu_pi+x*phi_pi')*100;
    zline = zeros(length(sam),1);

    curname = ['\beta = ' num2str(beta) ', \gamma = ' num2str(gam)];

  % models in the 4 state case: we get them only for cons., infl    
    % but one model with extra stuff    
        
    mnomchanges = ssmodel([mu_c;mu_pi;zeros(keepyields,1)],phi_x,sig_x,[eye(2) zeros(2,2);b(mats,:)*(phi_x-eye(Nstates))],...
                                [sigma(1:2,:);b(mats,:)*sig_x],eye(Nstates),curname);
    % baseline model: fundamentals + nominal short rate + nominal 5-1 spread                        
    mallyields = ssmodel([mu_c;mu_pi;a(mats);ar(mats)],phi_x,sig_x,...
                  [eye(2) zeros(2,2);[b(mats,:);br(mats,:)]*phi_x],...
                       [sigma(1:2,:);[b(mats,:);br(mats,:)]*sig_x],eye(Nstates),curname);
    mnomyields = ssmodel([mu_c;mu_pi;a(mats)],phi_x,sig_x,[eye(2) zeros(2,2);b(mats,:)*phi_x],[sigma(1:2,:);...
                            b(mats,:)*sig_x],eye(Nstates),curname);

    myields = ssmodel([mu_c;mu_pi;a(1);a(mats2)-a(1)*ones(keepyields2-1,1)],phi_x,sig_x,...
                  [eye(2) zeros(2,2);b(1,:)*phi_x;(b(mats2,:)-ones(keepyields2-1,1)*b(1,:))*phi_x],...
                       [sigma(1:2,:);b(1,:)*sig_x;(b(mats2,:)-ones(keepyields2-1,1)*b(1,:))*sig_x],eye(Nstates),curname);
    myields2 = ssmodel([mu_c;mu_pi;mu_y1;mu_spread;a(1);a(mats2)-a(1)*ones(keepyields2-1,1)],phi_x,sig_x,...
                  [eye(4);b(1,:)*phi_x;(b(mats2,:)-ones(keepyields2-1,1)*b(1,:))*phi_x],...
                       [sigma;b(1,:)*sig_x;(b(mats2,:)-ones(keepyields2-1,1)*b(1,:))*sig_x],eye(Nstates),curname);
    yieldmodels{jj} = myields;
    yieldmodels2{jj} = myields2;
    compyieldmodels{jj} = mnomyields;
    yieldchanges{jj} = mnomchanges;

    
end;

    
    ynomsA = ynoms*4;
    yrealsA =yreals*4;

    ynomA = squeeze(ynomsA(:,:,2));
    yrealA = squeeze(yrealsA(:,:,2));
    
    ydatA = ydat*4;  
    %------------------Autocorrelations -----------------------------
    
    for i = 1:size(ynomA,2)
       autocmodel(i) =  autoc(ynomA(:,i),1,0);
    end
    autocmodel = autocmodel(mats);
    for i = 1:6
        autocdata(i) = autoc(ydatA(:,i),1,0);
    end
    
    disp('Autocorrelations of nom yields: data, model');
    disp([autocdata; autocmodel])
    
    
    for i = 1:size(ynomA,2)
       autocmodel(i) =  autoc(yrealA(:,i),1,0);
    end
    autocmodel = autocmodel(mats);
   
    disp('Autocorrelations of real yields: data, model');
    disp([autocmodel])
    
    %------------------------------------------------------------
   
    
    
    disp('------------Real Yield Implications --------');
    yrealA = squeeze(yrealsA(:,:,1));
    disp('Mean and Volatility, first parameter constell')
    disp([mean(yrealA(:,mats)); std(yrealA(:,mats))]);
    
    yrealA = squeeze(yrealsA(:,:,2));

    disp('                      second parameter constell');
    disp([mean(yrealA(:,mats)); std(yrealA(:,mats))]);
    
    
    ynom = squeeze(ynoms(:,:,2));
    yreal = squeeze(yreals(:,:,2));
  
    ydatmodel= [yreal ynom];
    
    save ydatmodel.dat ydatmodel -ascii;
   

%-----------------Data on nominal yields------------------------------------%

%     load zc.dat;
%     ydat = zc(1:3:end,2:end); % quarterly data, starts 1952:2
%     ydat = ydat(3:4:end,:); % start 1952:4
%     date  =  (1952:2005)';
%    
    
    % baseline data set
    ydatA = ydat*4;
    spreaddat = ydat(:,6)-ydat(:,1);
    spreaddatA = spreaddat*4;
    dataA = data*4;
    basedata = [data (ydat(:,1)-mean(ydat(:,1))) (spreaddat-mean(spreaddat))];
    
    basedataA =[data*4 (ydat(:,1)-mean(ydat(:,1)))*4 (spreaddatA-mean(spreaddatA))];
    
    figure;
    subplot(2,2,1);
    plot(sam,dataA(:,1));
    title('cons growth');
    axis tight;
    subplot(2,2,2);
    plot(sam,dataA(:,2));
    axis tight;
    title('inflation');
    subplot(2,2,3);
    plot(sam,ydatA(:,[1 5]));
    legend('1 year','5 year');
    axis tight;
    subplot(2,2,4);
    plot(sam,[ydatA(:,1) spreaddatA]);
    legend('1 year','spread');
    plot(sam,zeros(T,1),'k-');

    axis tight;
    
    
    % covariance structure
    names{1} = '\Delta c';
    names{2} = '\pi';
    names{3} = 'y1';
    names{4} = 'y20';    
    names{5} = 'impl. y1';
    names{6} = 'impl. y20';    
    covplot(yieldmodels2{1}*100,basedata,names,10);
    test{1} = yieldmodels2{1}*100;
    corrplotn(test,basedata,names,10);
    
    
data = data(:,1:2);    
basedata = basedata(:,1:4);    
    
%----------------Compare Model and Data ---------------------------------%
disp('***************************************************************************');
disp('***************************************************************************');
disp('***************************************************************************');
disp(' =========================  BASELINE MODEL =============================');
    names{1} = '\Delta c';
    names{2} = '\pi';
    names{3} = 'y1';
    names{4} = 'y20';    

datameans = [mu_c*100,mu_pi*100,mean(ydat(:,1)), mean(spreaddat)];


summaryA(yieldmodels,basedata/100,datameans/100,names,3);
  

disp('***************************************************************************');
disp('***************************************************************************');
disp('***************************************************************************');
disp(' =========================  ALL NOMINAL YIELDS =============================');

% summary statistics; all yields
names{1} = 'cons. growth';
names{2} = 'inflation';
for j=1:keepyields2;
    names{j+2} = ['nominal ' num2str(mats(j))];
    %names{j+numyields+2} = ['real ' num2str(mats(j))];
end;
datameans = [mu_c*100,mu_pi*100,mean(ydat)];
datad = [data ydat-ones(T,1)*mean(ydat)];
summaryA(compyieldmodels,datad/100,datameans/100,names,3);

% table with levels of nominal and real yields
disp('***************************************************************************');
disp('***************************************************************************');
disp('***************************************************************************');
disp(' =========================  REAL TIME YIELDS =============================');
    legends{1}='data'; 
    disp('data: mean and standard deviations');
    disp([mean(ydatA);std(ydatA)]);
    disp('models: mean and standard deviations');
    for jj=1:numprefs;           
     mod = yieldmodels{jj};   
     disp(mod.name);
     legends{jj+1}=mod.name;
     disp('^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^');
     disp([mean(squeeze(ynomsA(:,mats,jj)));std(squeeze(ynomsA(:,mats,jj)))]);     
     disp('      REAL: mean and standard deviations');
     disp([mean(squeeze(yrealsA(:,mats,jj)));std(squeeze(yrealsA(:,mats,jj)))]);
    end;

    colors = 'bcrmyk';

% real time short rate and long rate pictures
  for matt = 1:5:6;
    figure;
    hold on;
    plot(sam, ydatA(:,matt),'g-','LineWidth',2);
    for jj=1:numprefs;
    style = [colors(jj) '-'];    
    plot(sam, squeeze(ynomsA(:,mats(matt),jj)),style,'LineWidth',2);
    end;
    for jj=1:numprefs;
    style = [colors(jj) '--'];    
    plot(sam, squeeze(yrealsA(:,mats(matt),jj)),style,'LineWidth',2);
    end;
    title(['maturity ' num2str(mats(matt)) ' (nominal solid, real dashed)']);
    legend(legends);
  end;
    
 
 % real time spread
    spreadnoms = squeeze(ynoms(:,20,:)) - squeeze(ynoms(:,1,:));
    spreadnomsA = spreadnoms*4;
    
    figure;
    hold on;
    plot(sam, spreaddatA,'g-','LineWidth',2);
    for jj=1:numprefs;
    style = [colors(jj) '-'];    
    plot(sam, spreadnomsA(:,jj),style,'LineWidth',2);
    end;
    title('nominal spread');
    legend(legends);
    plot(sam,zeros(T,1),'k-');

    figure;
    plot(sam, zline,'k', sam, ynomA(:,1)-yrealA(:,1), 'r', sam, Epi*4, 'b:');
    title('difference in yields and expected inflation');

%     figure; 
%     plot(sam, pgrowth*100,'k', yearF, piF*100,'r', sam+1, Epi,'r:', date-1, ydat(:,1), 'g', 'Linewidth',1.5);
%     % here, 1-year yield is pre-shifted, 
%     print -depsc2 fig1.eps;
%    
%     axis([1960 2005 1 14]);
%    
%     
    %figure; 
    %plot(yearF, piF(4:4:end)*100 - Epi,'Linewidth',1.5);

    %print -depsc2 fig2.eps;
   
    %axis([1960 2005 1 14]);

save ex2byields ynoms yreals;    
    
